@@ -97,8 +97,8 @@ class multi_type_vector
 public:
     typedef size_t size_type;
 
-    typedef typename mdds::mtv::base_element_block element_block_type;
-    typedef typename mdds::mtv::element_t element_category_type;
+    typedef mdds::mtv::base_element_block element_block_type;
+    typedef mdds::mtv::element_t element_category_type;
     typedef _ElemBlockFunc element_block_func;
 
     /**
@@ -124,12 +124,13 @@ private:
 
     struct block
     {
+        size_type m_position;
         size_type m_size;
         element_block_type* mp_data;
 
         block();
-        block(size_type _size);
-        block(size_type _size, element_block_type* _data);
+        block(size_type _position, size_type _size);
+        block(size_type _position, size_type _size, element_block_type* _data);
         block(const block& other);
         block(block&& other);
         ~block();
@@ -1070,6 +1071,8 @@ public:
 
 private:
 
+    void adjust_block_positions(int64_t start_block_index, int64_t delta);
+
     /**
      * Delete only the element block owned by an outer block.
      *
@@ -1088,55 +1091,46 @@ private:
     void delete_element_blocks(typename blocks_type::iterator it, typename blocks_type::iterator it_end);
 
     template<typename _T>
-    iterator set_impl(size_type pos, size_type start_row, size_type block_index, const _T& value);
+    iterator set_impl(size_type pos, size_type block_index, const _T& value);
 
     template<typename _T>
-    iterator release_impl(size_type pos, size_type start_pos, size_type block_index, _T& value);
+    iterator release_impl(size_type pos, size_type block_index, _T& value);
+
+    template<typename _T>
+    iterator push_back_impl(const _T& value);
 
     /**
-     * Find the correct block position for given logical row ID.
+     * Find the correct block position for a given logical row ID.
      *
-     * @param row logical ID of the row that belongs to the block being looked
-     *            up for.
+     * @param start_block_index index of the first block to start the search
+     *                          from.
      *
-     * @param start_pos logical ID of the first row of the block being looked
-     *                  up for. The caller needs to assign its initial value
-     *                  before calling this method in case the search needs to
-     *                  start with a block that's not the first block.  Assign
-     *                  0 if the search starts from the first block.
-     *
-     * @param block_index index of the block being looked up for. The caller
-     *                    needs to assign its initial index which will be the
-     *                    index of the block from which the search starts.
-     *                    Assign 0 if the search starts from the first block.
-     *
-     * @return true if block position is found, false otherwise.
+     * @return index of the block that contains the specified logical row ID.
      */
-    bool get_block_position(size_type row, size_type& start_pos, size_type& block_index) const;
+    size_type get_block_position(size_type row, size_type start_block_index=0) const;
 
     /**
      * Same as above, but try to infer block position from the iterator first
      * before trying full search.
      */
-    void get_block_position(const const_iterator& pos_hint, size_type pos, size_type& start_pos, size_type& block_index) const;
+    size_type get_block_position(const const_iterator& pos_hint, size_type row) const;
 
     template<typename _T>
     void create_new_block_with_new_cell(element_block_type*& data, const _T& cell);
 
     template<typename _T>
     iterator set_cell_to_middle_of_block(
-        size_type start_row, size_type block_index, size_type pos_in_block, const _T& cell);
+        size_type block_index, size_type pos_in_block, const _T& cell);
 
     template<typename _T>
     void append_cell_to_block(size_type block_index, const _T& cell);
 
     template<typename _T>
-    iterator set_cell_to_empty_block(
-        size_type start_row, size_type block_index, size_type pos_in_block, const _T& cell);
+    iterator set_cell_to_empty_block(size_type block_index, size_type pos_in_block, const _T& cell);
 
     template<typename _T>
     iterator set_cell_to_block_of_size_one(
-        size_type start_row, size_type block_index, const _T& cell);
+        size_type block_index, const _T& cell);
 
     template<typename _T>
     void set_cell_to_top_of_data_block(
@@ -1147,14 +1141,14 @@ private:
         size_type block_index, const _T& cell);
 
     iterator transfer_impl(
-        size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
+        size_type start_pos, size_type end_pos, size_type block_index1,
         multi_type_vector& dest, size_type dest_pos);
 
     /**
      * All elements to transfer to the other container is in the same block.
      */
     iterator transfer_single_block(
-        size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
+        size_type start_pos, size_type end_pos, size_type block_index1,
         multi_type_vector& dest, size_type dest_pos);
 
     /**
@@ -1162,53 +1156,69 @@ private:
      * blocks.
      */
     iterator transfer_multi_blocks(
-        size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
-        size_type start_pos_in_block2, size_type block_index2,
+        size_type start_pos, size_type end_pos, size_type block_index1, size_type block_index2,
         multi_type_vector& dest, size_type dest_pos);
 
+    /**
+     * @param start_pos logical start position.
+     * @param end_pos logical end position.
+     * @param start_pos_in_block1 logical position of the first element of the
+     *                            first block.
+     * @param block_index1 index of the first block
+     * @param overwrite when true, and when the stored values are pointers to
+     *                  heap objects, objects pointed to by the overwritten
+     *                  pointers should be freed from memory.
+     */
     iterator set_empty_impl(
-        size_type start_pos, size_type end_pos, size_type start_pos_in_block1, size_type block_index1,
-        bool overwrite);
+        size_type start_pos, size_type end_pos, size_type block_index1, bool overwrite);
 
     void swap_impl(
         multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
-        size_type start_pos_in_block1, size_type block_index1, size_type start_pos_in_block2, size_type block_index2,
-        size_type start_pos_in_dblock1, size_type dblock_index1, size_type start_pos_in_dblock2, size_type dblock_index2);
+        size_type block_index1, size_type block_index2, size_type dblock_index1, size_type dblock_index2);
 
     void swap_single_block(
         multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
-        size_type start_pos_in_block, size_type block_index, size_type start_pos_in_other_block, size_type other_block_index);
+        size_type block_index, size_type other_block_index);
 
     void swap_single_to_multi_blocks(
         multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
-        size_type start_pos_in_block, size_type block_index, size_type dst_start_pos_in_block1, size_type dst_block_index1,
-        size_type dst_start_pos_in_block2, size_type dst_block_index2);
+        size_type block_index, size_type dst_block_index1, size_type dst_block_index2);
 
     void swap_multi_to_multi_blocks(
         multi_type_vector& other, size_type start_pos, size_type end_pos, size_type other_pos,
-        size_type start_pos_in_block1, size_type block_index1, size_type start_pos_in_block2, size_type block_index2,
-        size_type start_pos_in_dblock1, size_type dblock_index1, size_type start_pos_in_dblock2, size_type dblock_index2);
+        size_type block_index1, size_type block_index2, size_type dblock_index1, size_type dblock_index2);
 
-    void insert_blocks_at(size_type insert_pos, blocks_type& new_blocks);
+    void insert_blocks_at(size_type position, size_type insert_pos, blocks_type& new_blocks);
 
     void prepare_blocks_to_transfer(blocks_to_transfer& bucket, size_type block_index1, size_type offset1, size_type block_index2, size_type offset2);
 
-    iterator set_whole_block_empty(size_type block_index, size_type start_pos_in_block, bool overwrite);
+    iterator set_whole_block_empty(size_type block_index, bool overwrite);
 
     iterator set_empty_in_single_block(
-        size_type start_pos, size_type end_pos, size_type block_index, size_type start_pos_in_block,
+        size_type start_row, size_type end_row, size_type block_index, bool overwrite);
+
+    /**
+     * @param start_pos logical start position.
+     * @param end_pos logical end position.
+     * @param block_index1 index of the first block.
+     * @param block_index2 index of the last block.
+     * @param overwrite when true, and when the stored values are pointers to
+     *                  heap objects, objects pointed to by the overwritten
+     *                  pointers should be freed from memory.
+     */
+    iterator set_empty_in_multi_blocks(
+        size_type start_row, size_type end_row, size_type block_index1, size_type block_index2,
         bool overwrite);
 
-    iterator set_empty_in_multi_blocks(
-        size_type start_pos, size_type end_pos,
-        size_type block_index1, size_type start_pos_in_block1,
-        size_type block_index2, size_type start_pos_in_block2, bool overwrite);
-
     void erase_impl(size_type start_pos, size_type end_pos);
-    void erase_in_single_block(
-        size_type start_pos, size_type end_pos, size_type block_pos, size_type start_pos_in_block);
+    void erase_in_single_block(size_type start_pos, size_type end_pos, size_type block_pos);
 
-    iterator insert_empty_impl(size_type row, size_type start_pos, size_type block_index, size_type length);
+    /**
+     * @param row logical position at which to insert an empty segment.
+     * @param block_index index of the block.
+     * @param length length of the emtpy segment to insert.
+     */
+    iterator insert_empty_impl(size_type pos, size_type block_index, size_type length);
 
     template<typename _T>
     bool set_cells_precheck(
@@ -1216,35 +1226,29 @@ private:
 
     template<typename _T>
     iterator set_cells_impl(
-        size_type row, size_type end_row, size_type start_row1, size_type block_index1, const _T& it_begin, const _T& it_end);
+        size_type row, size_type end_row, size_type block_index1, const _T& it_begin, const _T& it_end);
 
     template<typename _T>
-    iterator insert_cells_impl(size_type row, size_type start_row, size_type block_index, const _T& it_begin, const _T& it_end);
+    iterator insert_cells_impl(size_type row, size_type block_index, const _T& it_begin, const _T& it_end);
 
     template<typename _T>
     iterator set_cells_to_single_block(
-        size_type start_pos, size_type end_pos, size_type block_index,
-        size_type start_pos_in_block, const _T& it_begin, const _T& it_end);
+        size_type start_row, size_type end_row, size_type block_index,
+        const _T& it_begin, const _T& it_end);
 
     template<typename _T>
     iterator set_cells_to_multi_blocks(
-        size_type start_pos, size_type end_pos,
-        size_type block_index1, size_type start_pos_in_block1,
-        size_type block_index2, size_type start_pos_in_block2,
+        size_type start_row, size_type end_row, size_type block_index1, size_type block_index2,
         const _T& it_begin, const _T& it_end);
 
     template<typename _T>
     iterator set_cells_to_multi_blocks_block1_non_equal(
-        size_type start_pos, size_type end_pos,
-        size_type block_index1, size_type start_pos_in_block1,
-        size_type block_index2, size_type start_pos_in_block2,
+        size_type start_row, size_type end_row, size_type block_index1, size_type block_index2,
         const _T& it_begin, const _T& it_end);
 
     template<typename _T>
     iterator set_cells_to_multi_blocks_block1_non_empty(
-        size_type start_pos, size_type end_pos,
-        size_type block_index1, size_type start_pos_in_block1,
-        size_type block_index2, size_type start_pos_in_block2,
+        size_type start_row, size_type end_row, size_type block_index1, size_type block_index2,
         const _T& it_begin, const _T& it_end);
 
     /**
@@ -1273,8 +1277,7 @@ private:
 
     template<typename _T>
     void insert_cells_to_middle(
-        size_type row, size_type block_index, size_type start_pos,
-        const _T& it_begin, const _T& it_end);
+        size_type row, size_type block_index, const _T& it_begin, const _T& it_end);
 
     /**
      * Set a new block in the middle of an existing block. This call inserts
@@ -1330,18 +1333,18 @@ private:
 
     bool append_empty(size_type len);
 
-    inline iterator get_iterator(size_type block_index, size_type start_row)
+    inline iterator get_iterator(size_type block_index)
     {
         typename blocks_type::iterator block_pos = m_blocks.begin();
         std::advance(block_pos, block_index);
-        return iterator(block_pos, m_blocks.end(), start_row, block_index);
+        return iterator(block_pos, m_blocks.end(), block_index);
     }
 
-    inline const_iterator get_const_iterator(size_type block_index, size_type start_row) const
+    inline const_iterator get_const_iterator(size_type block_index) const
     {
         typename blocks_type::const_iterator block_pos = m_blocks.begin();
         std::advance(block_pos, block_index);
-        return const_iterator(block_pos, m_blocks.end(), start_row, block_index);
+        return const_iterator(block_pos, m_blocks.end(), block_index);
     }
 
 private:
