@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * Copyright (c) 2016 Kohei Yoshida
+ * Copyright (c) 2016-2020 Kohei Yoshida
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -36,6 +36,7 @@
 #endif
 
 #include "mdds/global.hpp"
+#include "mdds/ref_pair.hpp"
 
 namespace mdds { namespace trie { namespace detail {
 
@@ -62,45 +63,68 @@ enum class iterator_type
 
 enum empty_iterator_type { empty_iterator };
 
+template<typename _TrieType, typename _C>
+struct get_node_stack_type;
+
+template<typename _TrieType>
+struct get_node_stack_type<_TrieType, std::true_type>
+{
+    using type = typename _TrieType::const_node_stack_type;
+};
+
+template<typename _TrieType>
+struct get_node_stack_type<_TrieType, std::false_type>
+{
+    using type = typename _TrieType::node_stack_type;
+};
+
 template<typename _TrieType>
 class search_results;
 
-template<typename _TrieType>
+template<typename _TrieType, bool _IsConst>
 class iterator_base
 {
-    typedef _TrieType trie_type;
+protected:
+    using trie_type = _TrieType;
+
+    using _is_const = bool_constant<_IsConst>;
+
     friend trie_type;
     friend search_results<trie_type>;
 
-    typedef typename trie_type::node_stack_type node_stack_type;
+    using node_stack_type = typename get_node_stack_type<trie_type, _is_const>::type;
+    using trie_node_type = const_t<typename trie_type::trie_node, _IsConst>;
+    using trie_node_child_pos_type =
+        typename get_iterator_type<
+            typename std::remove_const<trie_node_type>::type::children_type, _is_const>::type;
 
-    typedef typename trie_type::trie_node trie_node;
-    typedef typename trie_type::key_trait_type key_trait_type;
-    typedef typename key_trait_type::key_type key_type;
-    typedef typename key_trait_type::key_buffer_type key_buffer_type;
-    typedef typename key_trait_type::key_unit_type   key_unit_type;
+    using key_trait_type = typename trie_type::key_trait_type;
+    using key_type = typename key_trait_type::key_type;
+    using key_buffer_type = typename key_trait_type::key_buffer_type;
+    using trie_value_type = typename const_or_not<typename trie_type::value_type, _is_const>::type;
 
 public:
     // iterator traits
-    typedef typename trie_type::key_value_type value_type;
-    typedef value_type*     pointer;
-    typedef value_type&     reference;
-    typedef std::ptrdiff_t  difference_type;
-    typedef std::bidirectional_iterator_tag iterator_category;
+    using value_type = mdds::detail::ref_pair<typename std::add_const<key_type>::type, trie_value_type>;
+    using pointer = value_type*;
+    using reference = value_type&;
+    using difference_type = std::ptrdiff_t;
+    using iterator_category = std::bidirectional_iterator_tag;
 
-private:
+protected:
     node_stack_type m_node_stack;
     key_buffer_type m_buffer;
-    value_type m_current_value;
+    key_type m_current_key;
+    trie_value_type* m_current_value_ptr;
     iterator_type m_type;
 
-    static const trie_node* push_child_node_to_stack(
+    static trie_node_type* push_child_node_to_stack(
         node_stack_type& node_stack, key_buffer_type& buf,
-        const typename trie_node::children_type::const_iterator& child_pos)
+        trie_node_child_pos_type& child_pos)
     {
         using ktt = key_trait_type;
 
-        const trie_node* node = &child_pos->second;
+        trie_node_type* node = &child_pos->second;
         ktt::push_back(buf, child_pos->first);
         node_stack.emplace_back(node, node->children.begin());
 
@@ -111,12 +135,12 @@ private:
      * From the current node, move to its previous child node and descend all
      * the way to the leaf node.
      */
-    static const trie_node* descend_to_previus_leaf_node(
+    static trie_node_type* descend_to_previus_leaf_node(
         node_stack_type& node_stack, key_buffer_type& buf)
     {
         using ktt = key_trait_type;
 
-        const trie_node* cur_node = nullptr;
+        trie_node_type* cur_node = nullptr;
 
         do
         {
@@ -143,7 +167,8 @@ public:
     iterator_base(node_stack_type&& node_stack, key_buffer_type&& buf, iterator_type type) :
         m_node_stack(std::move(node_stack)),
         m_buffer(std::move(buf)),
-        m_current_value(key_trait_type::to_key(m_buffer), m_node_stack.back().node->value),
+        m_current_key(key_trait_type::to_key(m_buffer)),
+        m_current_value_ptr(&m_node_stack.back().node->value),
         m_type(type)
     {}
 
@@ -163,21 +188,21 @@ public:
         return !operator==(other);
     }
 
-    const value_type& operator*()
+    value_type operator*()
     {
-        return m_current_value;
+        return value_type(m_current_key, *m_current_value_ptr);
     }
 
-    const value_type* operator->()
+    value_type operator->()
     {
-        return &m_current_value;
+        return value_type(m_current_key, *m_current_value_ptr);
     }
 
     iterator_base& operator++()
     {
         using ktt = key_trait_type;
 
-        const trie_node* cur_node = m_node_stack.back().node;
+        trie_node_type* cur_node = m_node_stack.back().node;
 
         do
         {
@@ -227,7 +252,8 @@ public:
         }
         while (!cur_node->has_value);
 
-        m_current_value = value_type(ktt::to_key(m_buffer), cur_node->value);
+        m_current_key = ktt::to_key(m_buffer);
+        m_current_value_ptr = &cur_node->value;
         return *this;
     }
 
@@ -241,7 +267,7 @@ public:
     iterator_base& operator--()
     {
         using ktt = key_trait_type;
-        const trie_node* cur_node = m_node_stack.back().node;
+        trie_node_type* cur_node = m_node_stack.back().node;
 
         if (m_type == iterator_type::end && cur_node->has_value)
         {
@@ -308,7 +334,8 @@ public:
         }
 
         assert(cur_node->has_value);
-        m_current_value = value_type(ktt::to_key(m_buffer), cur_node->value);
+        m_current_key = ktt::to_key(m_buffer);
+        m_current_value_ptr = &cur_node->value;
         return *this;
     }
 
@@ -320,18 +347,107 @@ public:
     }
 };
 
+template<typename _TrieType>
+class const_iterator;
+
+template<typename _TrieType>
+class iterator : public iterator_base<_TrieType, false>
+{
+    using trie_type = _TrieType;
+
+    friend trie_type;
+    friend search_results<trie_type>;
+    friend const_iterator<trie_type>;
+
+    using base_type = iterator_base<trie_type, false>;
+    using node_stack_type = typename base_type::node_stack_type;
+    using key_buffer_type = typename base_type::key_buffer_type;
+
+    iterator(empty_iterator_type t) : base_type(t) {}
+
+public:
+    iterator() : base_type() {}
+
+    iterator(node_stack_type&& node_stack, key_buffer_type&& buf, iterator_type type) :
+        base_type(std::move(node_stack), std::move(buf), type)
+    {}
+};
+
+template<typename _TrieType>
+class const_iterator : public iterator_base<_TrieType, true>
+{
+    using trie_type = _TrieType;
+
+    friend trie_type;
+    friend search_results<trie_type>;
+
+    using base_type = iterator_base<trie_type, true>;
+    using node_stack_type = typename base_type::node_stack_type;
+    using key_buffer_type = typename base_type::key_buffer_type;
+
+    using base_type::m_node_stack;
+    using base_type::m_buffer;
+    using base_type::m_current_key;
+    using base_type::m_current_value_ptr;
+    using base_type::m_type;
+
+    const_iterator(empty_iterator_type t) : base_type(t) {}
+
+public:
+    const_iterator() : base_type() {}
+
+    const_iterator(node_stack_type&& node_stack, key_buffer_type&& buf, iterator_type type) :
+        base_type(std::move(node_stack), std::move(buf), type)
+    {}
+
+    const_iterator(const iterator<_TrieType>& it) : base_type()
+    {
+        m_buffer = it.m_buffer;
+        m_current_key = it.m_current_key;
+        m_current_value_ptr = it.m_current_value_ptr;
+        m_type = it.m_type;
+
+        for (const auto& e : it.m_node_stack)
+            m_node_stack.emplace_back(e.node, e.child_pos);
+    }
+};
+
+template<typename _TrieType>
+bool operator==(const iterator<_TrieType>& left, const const_iterator<_TrieType>& right)
+{
+    return const_iterator<_TrieType>(left) == right;
+}
+
+template<typename _TrieType>
+bool operator!=(const iterator<_TrieType>& left, const const_iterator<_TrieType>& right)
+{
+    return const_iterator<_TrieType>(left) != right;
+}
+
+template<typename _TrieType>
+bool operator==(const const_iterator<_TrieType>& left, const iterator<_TrieType>& right)
+{
+    return left == const_iterator<_TrieType>(right);
+}
+
+template<typename _TrieType>
+bool operator!=(const const_iterator<_TrieType>& left, const iterator<_TrieType>& right)
+{
+    return left != const_iterator<_TrieType>(right);
+}
+
 template<typename _TrieType>
 class search_results
 {
-    typedef _TrieType trie_type;
+    using trie_type = _TrieType;
     friend trie_type;
-    typedef typename trie_type::node_stack_type node_stack_type;
+    using node_stack_type = typename trie_type::const_node_stack_type;
 
-    typedef typename trie_type::trie_node trie_node;
-    typedef typename trie_type::key_trait_type key_trait_type;
-    typedef typename key_trait_type::key_type key_type;
-    typedef typename key_trait_type::key_buffer_type key_buffer_type;
-    typedef typename key_trait_type::key_unit_type   key_unit_type;
+    using trie_node = typename trie_type::trie_node;
+    using key_trait_type = typename trie_type::key_trait_type;
+    using key_type = typename key_trait_type::key_type;
+    using key_buffer_type = typename key_trait_type::key_buffer_type;
+    using key_unit_type = typename key_trait_type::key_unit_type;
 
     const trie_node* m_node;
     key_buffer_type m_buffer;
@@ -341,7 +457,7 @@ class search_results
         m_node(node), m_buffer(buf) {}
 
 public:
-    typedef iterator_base<trie_type> const_iterator;
+    using const_iterator = typename trie_type::const_iterator;
 
     const_iterator begin() const
     {
@@ -386,25 +502,25 @@ class packed_search_results;
 template<typename _TrieType>
 class packed_iterator_base
 {
-    typedef _TrieType trie_type;
+    using trie_type = _TrieType;
     friend trie_type;
     friend packed_search_results<trie_type>;
 
-    typedef typename trie_type::stack_item stack_item;
-    typedef typename trie_type::node_stack_type node_stack_type;
+    using stack_item = typename trie_type::stack_item;
+    using node_stack_type = typename trie_type::node_stack_type;
 
-    typedef typename trie_type::key_trait_type key_trait_type;
-    typedef typename key_trait_type::key_type key_type;
-    typedef typename key_trait_type::key_buffer_type key_buffer_type;
-    typedef typename key_trait_type::key_unit_type   key_unit_type;
+    using key_trait_type = typename trie_type::key_trait_type;
+    using key_type = typename key_trait_type::key_type;
+    using key_buffer_type = typename key_trait_type::key_buffer_type;
+    using key_unit_type = typename key_trait_type::key_unit_type;
 
 public:
     // iterator traits
-    typedef typename trie_type::key_value_type value_type;
-    typedef value_type*     pointer;
-    typedef value_type&     reference;
-    typedef std::ptrdiff_t  difference_type;
-    typedef std::bidirectional_iterator_tag iterator_category;
+    using value_type = typename trie_type::key_value_type;
+    using pointer = value_type*;
+    using reference = value_type&;
+    using difference_type = std::ptrdiff_t;
+    using iterator_category = std::bidirectional_iterator_tag;
 
 private:
     node_stack_type m_node_stack;
@@ -687,14 +803,14 @@ public:
 template<typename _TrieType>
 class packed_search_results
 {
-    typedef _TrieType trie_type;
+    using trie_type = _TrieType;
     friend trie_type;
-    typedef typename trie_type::node_stack_type node_stack_type;
+    using node_stack_type = typename trie_type::node_stack_type;
 
-    typedef typename trie_type::key_trait_type key_trait_type;
-    typedef typename key_trait_type::key_type key_type;
-    typedef typename key_trait_type::key_buffer_type key_buffer_type;
-    typedef typename key_trait_type::key_unit_type   key_unit_type;
+    using key_trait_type = typename trie_type::key_trait_type;
+    using key_type = typename key_trait_type::key_type;
+    using key_buffer_type = typename key_trait_type::key_buffer_type;
+    using key_unit_type = typename key_trait_type::key_unit_type;
 
     const uintptr_t* m_node;
     key_buffer_type m_buffer;
@@ -724,7 +840,7 @@ class packed_search_results
     }
 
 public:
-    typedef packed_iterator_base<trie_type> const_iterator;
+    using const_iterator = packed_iterator_base<trie_type>;
 
     packed_search_results() : m_node(nullptr) {}
 
